@@ -9,11 +9,8 @@
 #include <linux/edac.h>
 #include <linux/module.h>
 #include <linux/platform_device.h>
-#include <linux/spinlock.h>
-#include <linux/sizes.h>
 #include <linux/interrupt.h>
 #include <linux/of.h>
-#include <linux/of_device.h>
 
 #include "edac_module.h"
 
@@ -302,7 +299,6 @@ struct synps_ecc_status {
 /**
  * struct synps_edac_priv - DDR memory controller private instance data.
  * @baseaddr:		Base address of the DDR controller.
- * @reglock:		Concurrent CSRs access lock.
  * @message:		Buffer for framing the event specific info.
  * @stat:		ECC status information.
  * @p_data:		Platform data.
@@ -317,7 +313,6 @@ struct synps_ecc_status {
  */
 struct synps_edac_priv {
 	void __iomem *baseaddr;
-	spinlock_t reglock;
 	char message[SYNPS_EDAC_MSG_SIZE];
 	struct synps_ecc_status stat;
 	const struct synps_platform_data *p_data;
@@ -339,7 +334,6 @@ struct synps_edac_priv {
  * @get_mtype:		Get mtype.
  * @get_dtype:		Get dtype.
  * @get_ecc_state:	Get ECC state.
- * @get_mem_info:	Get EDAC memory info
  * @quirks:		To differentiate IPs.
  */
 struct synps_platform_data {
@@ -347,9 +341,6 @@ struct synps_platform_data {
 	enum mem_type (*get_mtype)(const void __iomem *base);
 	enum dev_type (*get_dtype)(const void __iomem *base);
 	bool (*get_ecc_state)(void __iomem *base);
-#ifdef CONFIG_EDAC_DEBUG
-	u64 (*get_mem_info)(struct synps_edac_priv *priv);
-#endif
 	int quirks;
 };
 
@@ -408,25 +399,6 @@ out:
 	return 0;
 }
 
-#ifdef CONFIG_EDAC_DEBUG
-/**
- * zynqmp_get_mem_info - Get the current memory info.
- * @priv:	DDR memory controller private instance data.
- *
- * Return: host interface address.
- */
-static u64 zynqmp_get_mem_info(struct synps_edac_priv *priv)
-{
-	u64 hif_addr = 0, linear_addr;
-
-	linear_addr = priv->poison_addr;
-	if (linear_addr >= SZ_32G)
-		linear_addr = linear_addr - SZ_32G + SZ_2G;
-	hif_addr = linear_addr >> 3;
-	return hif_addr;
-}
-#endif
-
 /**
  * zynqmp_get_error_info - Get the current ECC error info.
  * @priv:	DDR memory controller private instance data.
@@ -436,8 +408,7 @@ static u64 zynqmp_get_mem_info(struct synps_edac_priv *priv)
 static int zynqmp_get_error_info(struct synps_edac_priv *priv)
 {
 	struct synps_ecc_status *p;
-	u32 regval, clearval;
-	unsigned long flags;
+	u32 regval, clearval = 0;
 	void __iomem *base;
 
 	base = priv->baseaddr;
@@ -481,14 +452,10 @@ ue_err:
 	p->ueinfo.blknr = (regval & ECC_CEADDR1_BLKNR_MASK);
 	p->ueinfo.data = readl(base + ECC_UESYND0_OFST);
 out:
-	spin_lock_irqsave(&priv->reglock, flags);
-
-	clearval = readl(base + ECC_CLR_OFST) |
-		   ECC_CTRL_CLR_CE_ERR | ECC_CTRL_CLR_CE_ERRCNT |
-		   ECC_CTRL_CLR_UE_ERR | ECC_CTRL_CLR_UE_ERRCNT;
+	clearval = ECC_CTRL_CLR_CE_ERR | ECC_CTRL_CLR_CE_ERRCNT;
+	clearval |= ECC_CTRL_CLR_UE_ERR | ECC_CTRL_CLR_UE_ERRCNT;
 	writel(clearval, base + ECC_CLR_OFST);
-
-	spin_unlock_irqrestore(&priv->reglock, flags);
+	writel(0x0, base + ECC_CLR_OFST);
 
 	return 0;
 }
@@ -548,41 +515,24 @@ static void handle_error(struct mem_ctl_info *mci, struct synps_ecc_status *p)
 
 static void enable_intr(struct synps_edac_priv *priv)
 {
-	unsigned long flags;
-
 	/* Enable UE/CE Interrupts */
-	if (!(priv->p_data->quirks & DDR_ECC_INTR_SELF_CLEAR)) {
+	if (priv->p_data->quirks & DDR_ECC_INTR_SELF_CLEAR)
+		writel(DDR_UE_MASK | DDR_CE_MASK,
+		       priv->baseaddr + ECC_CLR_OFST);
+	else
 		writel(DDR_QOSUE_MASK | DDR_QOSCE_MASK,
 		       priv->baseaddr + DDR_QOS_IRQ_EN_OFST);
 
-		return;
-	}
-
-	spin_lock_irqsave(&priv->reglock, flags);
-
-	writel(DDR_UE_MASK | DDR_CE_MASK,
-	       priv->baseaddr + ECC_CLR_OFST);
-
-	spin_unlock_irqrestore(&priv->reglock, flags);
 }
 
 static void disable_intr(struct synps_edac_priv *priv)
 {
-	unsigned long flags;
-
 	/* Disable UE/CE Interrupts */
-	if (!(priv->p_data->quirks & DDR_ECC_INTR_SELF_CLEAR)) {
+	if (priv->p_data->quirks & DDR_ECC_INTR_SELF_CLEAR)
+		writel(0x0, priv->baseaddr + ECC_CLR_OFST);
+	else
 		writel(DDR_QOSUE_MASK | DDR_QOSCE_MASK,
 		       priv->baseaddr + DDR_QOS_IRQ_DB_OFST);
-
-		return;
-	}
-
-	spin_lock_irqsave(&priv->reglock, flags);
-
-	writel(0, priv->baseaddr + ECC_CLR_OFST);
-
-	spin_unlock_irqrestore(&priv->reglock, flags);
 }
 
 /**
@@ -626,6 +576,8 @@ static irqreturn_t intr_handler(int irq, void *dev_id)
 	/* v3.0 of the controller does not have this register */
 	if (!(priv->p_data->quirks & DDR_ECC_INTR_SELF_CLEAR))
 		writel(regval, priv->baseaddr + DDR_QOS_IRQ_STAT_OFST);
+	else
+		enable_intr(priv);
 
 	return IRQ_HANDLED;
 }
@@ -947,9 +899,6 @@ static const struct synps_platform_data zynqmp_edac_def = {
 	.get_mtype	= zynqmp_get_mtype,
 	.get_dtype	= zynqmp_get_dtype,
 	.get_ecc_state	= zynqmp_get_ecc_state,
-#ifdef CONFIG_EDAC_DEBUG
-	.get_mem_info	= zynqmp_get_mem_info,
-#endif
 	.quirks         = (DDR_ECC_INTR_SUPPORT
 #ifdef CONFIG_EDAC_DEBUG
 			  | DDR_ECC_DATA_POISON_SUPPORT
@@ -1003,16 +952,10 @@ MODULE_DEVICE_TABLE(of, synps_edac_match);
 static void ddr_poison_setup(struct synps_edac_priv *priv)
 {
 	int col = 0, row = 0, bank = 0, bankgrp = 0, rank = 0, regval;
-	const struct synps_platform_data *p_data;
 	int index;
 	ulong hif_addr = 0;
 
-	p_data = priv->p_data;
-
-	if (p_data->get_mem_info)
-		hif_addr = p_data->get_mem_info(priv);
-	else
-		hif_addr = priv->poison_addr >> 3;
+	hif_addr = priv->poison_addr >> 3;
 
 	for (index = 0; index < DDR_MAX_ROW_SHIFT; index++) {
 		if (priv->row_shift[index])
@@ -1416,7 +1359,6 @@ static int mc_probe(struct platform_device *pdev)
 	priv = mci->pvt_info;
 	priv->baseaddr = baseaddr;
 	priv->p_data = p_data;
-	spin_lock_init(&priv->reglock);
 
 	mc_init(mci, pdev);
 
