@@ -31,7 +31,6 @@ struct i2c_hid_of_elan {
 	struct regulator *vcc33;
 	struct regulator *vccio;
 	struct gpio_desc *reset_gpio;
-	bool no_reset_on_power_off;
 	const struct elan_i2c_hid_chip_data *chip_data;
 };
 
@@ -41,17 +40,17 @@ static int elan_i2c_hid_power_up(struct i2chid_ops *ops)
 		container_of(ops, struct i2c_hid_of_elan, ops);
 	int ret;
 
-	gpiod_set_value_cansleep(ihid_elan->reset_gpio, 1);
-
 	if (ihid_elan->vcc33) {
 		ret = regulator_enable(ihid_elan->vcc33);
 		if (ret)
-			goto err_deassert_reset;
+			return ret;
 	}
 
 	ret = regulator_enable(ihid_elan->vccio);
-	if (ret)
-		goto err_disable_vcc33;
+	if (ret) {
+		regulator_disable(ihid_elan->vcc33);
+		return ret;
+	}
 
 	if (ihid_elan->chip_data->post_power_delay_ms)
 		msleep(ihid_elan->chip_data->post_power_delay_ms);
@@ -61,15 +60,6 @@ static int elan_i2c_hid_power_up(struct i2chid_ops *ops)
 		msleep(ihid_elan->chip_data->post_gpio_reset_on_delay_ms);
 
 	return 0;
-
-err_disable_vcc33:
-	if (ihid_elan->vcc33)
-		regulator_disable(ihid_elan->vcc33);
-err_deassert_reset:
-	if (ihid_elan->no_reset_on_power_off)
-		gpiod_set_value_cansleep(ihid_elan->reset_gpio, 0);
-
-	return ret;
 }
 
 static void elan_i2c_hid_power_down(struct i2chid_ops *ops)
@@ -77,14 +67,7 @@ static void elan_i2c_hid_power_down(struct i2chid_ops *ops)
 	struct i2c_hid_of_elan *ihid_elan =
 		container_of(ops, struct i2c_hid_of_elan, ops);
 
-	/*
-	 * Do not assert reset when the hardware allows for it to remain
-	 * deasserted regardless of the state of the (shared) power supply to
-	 * avoid wasting power when the supply is left on.
-	 */
-	if (!ihid_elan->no_reset_on_power_off)
-		gpiod_set_value_cansleep(ihid_elan->reset_gpio, 1);
-
+	gpiod_set_value_cansleep(ihid_elan->reset_gpio, 1);
 	if (ihid_elan->chip_data->post_gpio_reset_off_delay_ms)
 		msleep(ihid_elan->chip_data->post_gpio_reset_off_delay_ms);
 
@@ -93,11 +76,9 @@ static void elan_i2c_hid_power_down(struct i2chid_ops *ops)
 		regulator_disable(ihid_elan->vcc33);
 }
 
-static int i2c_hid_of_elan_probe(struct i2c_client *client,
-				 const struct i2c_device_id *id)
+static int i2c_hid_of_elan_probe(struct i2c_client *client)
 {
 	struct i2c_hid_of_elan *ihid_elan;
-	int ret;
 
 	ihid_elan = devm_kzalloc(&client->dev, sizeof(*ihid_elan), GFP_KERNEL);
 	if (!ihid_elan)
@@ -112,38 +93,21 @@ static int i2c_hid_of_elan_probe(struct i2c_client *client,
 	if (IS_ERR(ihid_elan->reset_gpio))
 		return PTR_ERR(ihid_elan->reset_gpio);
 
-	ihid_elan->no_reset_on_power_off = of_property_read_bool(client->dev.of_node,
-						"no-reset-on-power-off");
-
 	ihid_elan->vccio = devm_regulator_get(&client->dev, "vccio");
-	if (IS_ERR(ihid_elan->vccio)) {
-		ret = PTR_ERR(ihid_elan->vccio);
-		goto err_deassert_reset;
-	}
+	if (IS_ERR(ihid_elan->vccio))
+		return PTR_ERR(ihid_elan->vccio);
 
 	ihid_elan->chip_data = device_get_match_data(&client->dev);
 
 	if (ihid_elan->chip_data->main_supply_name) {
 		ihid_elan->vcc33 = devm_regulator_get(&client->dev,
 						      ihid_elan->chip_data->main_supply_name);
-		if (IS_ERR(ihid_elan->vcc33)) {
-			ret = PTR_ERR(ihid_elan->vcc33);
-			goto err_deassert_reset;
-		}
+		if (IS_ERR(ihid_elan->vcc33))
+			return PTR_ERR(ihid_elan->vcc33);
 	}
 
-	ret = i2c_hid_core_probe(client, &ihid_elan->ops,
-				 ihid_elan->chip_data->hid_descriptor_address, 0);
-	if (ret)
-		goto err_deassert_reset;
-
-	return 0;
-
-err_deassert_reset:
-	if (ihid_elan->no_reset_on_power_off)
-		gpiod_set_value_cansleep(ihid_elan->reset_gpio, 0);
-
-	return ret;
+	return i2c_hid_core_probe(client, &ihid_elan->ops,
+				  ihid_elan->chip_data->hid_descriptor_address, 0);
 }
 
 static const struct elan_i2c_hid_chip_data elan_ekth6915_chip_data = {
